""".. _RequestObject:

==========================
``yaro`` -- Request Object
==========================

This request object generating component uses
Yaro_ to provide a (hopefully) tidy API to 
implement your application against. In addition
to Yaro's usual attributes, the request object
provides the following:

=============== ==============================
``session``     Dict of session data.
``logger``      The application logger.
``cache``       The cache manager for the app.
``loc``         Application locations.
``ff``          Dict of framework functions.
``config``      The application config.
``args``        WSGI routing args.
``vars``        WSGI routing kwargs.
=============== ==============================

"""

from yaro import Yaro

from haus.components.abstract import Component, update_wrapper


class YaroComponent(Component):

    consumes = [
        'get_session', 
        'get_cache', 
        'get_logger', 
        'get_locaitons',
        'routing_args', 
        'routing_kwargs',
    ]

    def __call__(self, wrk, *args, **kwargs):
        def middleware(app):
            new = Yaro(app, [('session', wrk.functions['get_session']),
                             ('logger', wrk.functions['get_logger']),
                             ('cache', wrk.functions['get_cache']),
                             ('loc', wrk.functions['get_locations']),
                             ('ff', lambda e: wrk.functions),
                             ('config', lambda e: wrk.config),
                             ('args', wrk.functions['routing_args']),
                             ('vars', wrk.functions['routing_kwargs'])])
            new.__name__ = app.__name__
            return new
        return middleware



""".. _LoggingComponent:

``logger`` -- Application Logging
=================================

Provide a logger. Goes to ``sys.stderr``.

TODO: Expand the configuration options and stick
to standard library options as much as possible.

"""

import sys, logging

from haus.components.abstract import Component, wraps, update_wrapper


class LoggingComponent(Component):

    provides = ['get_logger']
    consumes = ['get_logger', 'status500']

    def __init__(self, wrk):
        self.package_name = wrk.package_name
        self.logger = None
        config = wrk.config.get('logging', {})
        level = config.get('level', 'DEBUG')
        self.level = getattr(logging, level)
        default_format = "[%(name)s:%(levelname)s:%(module)s][%(asctime)s] %(message)s"
        self.format = config.get('format', default_format)
        wrk.functions['get_logger'] = self.get_logger

    def get_logger(self, environ):
        if self.logger is None:
            logger = logging.getLogger(self.package_name)
            logger.setLevel(self.level)
            if logger.handlers == []:
                handler = logging.StreamHandler(sys.stderr)
                handler.setLevel(self.level)
                if self.format is not None:
                    formatter = logging.Formatter(self.format)
                    handler.setFormatter(formatter)
                logger.addHandler(handler)
            self.logger = logger
        return self.logger

    def __call__(self, wrk, *args, **kwargs):
        def middleware(app):
            @wraps(app)
            def proxy(environ, start_response):
                logger = wrk.functions['get_logger'](environ)
                start_response_called = [False]
                def new_start_response(a, b, c=None):
                    start_response_called[0] = True
                    return start_response(a, b, c)
                update_wrapper(new_start_response, start_response)
                try:
                    return app(environ, new_start_response)
                except Exception, e:
                    logger.exception(e)
                    if not start_response_called[0]:
                        return wrk.functions['status500'](environ, start_response)
                    else:
                        return wrk.functions['status500'](
                            environ, 
                            lambda a, b, c=None: None
                        )
            return proxy
        return middleware



""".. _BarrelSecurityComponent:

``barrel`` -- Authentication and Authorization
==============================================

**Component name:** ``barrel``

Provides basic and form based auth.

TODO: Describe all sorts of ways to customize this.
TODO: How should this be exposed more cleanly? User object?
"""

from pkg_resources import Requirement, resource_filename

from barrel import basic, form, combo, roles, cooper

from haus.components.abstract import Component, update_wrapper
from haus.core import tack

class BarrelSecurityComponent(Component):

    consumes = ['invalidate_session', 'get_session']
    provides = ['valid_user', 'user_roles', 'get_user', 
                'login', 'logout', 'login_page', 'logout_page',
                'need_auth', 'denied']

    def __init__(self, wrk):
        """."""
        session_config = wrk.config.get('sessions', {})
        self.session_key = session_config.get('environ_key')
        config = wrk.config.get('barrel', {})
        self.user_key = config.get('user_key', wrk.package_name + ".user")
        locations = wrk.functions['get_locations']({})
        prefix = wrk.config['app']['prefix']
        self.login_location = locations.get('login', prefix + '/login/')
        self.landing_location = locations.get('landing', prefix + '/')
        self.realm = config.get('realm', wrk.package_name)
        self.secrets = wrk.config.get('user.secrets', {})
        self.roles = wrk.config.get('user.roles', {})
        wrk.functions['valid_user'] = self.valid_user
        wrk.functions['user_roles'] = self.user_roles
        wrk.functions['get_user'] = self.get_cached_username
        wrk.functions['login'] = self.cache_username
        wrk.functions['logout'] = self.logout
        wrk.functions['login_page'] = self.login_page
        wrk.functions['logout_page'] = self.logout_page
        self.init_authn(wrk)
        self.init_authz(wrk)
        self.init_templates(wrk)

    def init_templates(self, wrk):
        # TODO: These could also be tuples of package, relative_path
        #       to enable template sharing via pkg_resources?
        # For now, relative path is in this package, otherwise absolute.
        config = wrk.config.get('barrel.templates', {})
        haus_resource = wrk.haus_resource
        self.template_login = config.get(
            'login',
            resource_filename(haus_resource, 'haus/html/login.html')
        )
        self.template_logout = config.get(
            'logout',
            resource_filename(haus_resource, 'haus/html/logout.html')
        )
        self.template_denied = config.get(
            'denied',
            resource_filename(haus_resource, 'haus/html/denied.html')
        )
        self.layout = config.get(
            'layout',
            wrk.pdfilename('html/_layouts/default.html')
        )


    def __call__(self, wrk, *args, **kwargs):
        """."""
        wrk.functions['need_auth'] = wrk.stackify(self.need_auth)
        wrk.functions['denied'] = wrk.stackify(self.access_denied)

        #auth_options = copy(wrk.config['barrel'])
        # for auth options see barrel docs
        config = wrk.config.get('barrel', {})
        auth_options = dict(
            realm=config.get('realm', wrk.package_name),
        )
        auth_options.update(kwargs)
        authn = self.authn(**auth_options)
        authz = self.authz(**auth_options)

        def middleware(app):
            newapp = authn(authz(app))
            return update_wrapper(newapp, app)

        return middleware

    def valid_user(self, username, password):
        return username and password and \
            self.secrets.get(username) == password

    def security_mixin_factory(self, wrk):
        """Return a class that implements user validation."""
        outer = self

        class SecurityMixIn(object):
            """From haus.factories.BarrelSecurityFactory."""
            def valid_user(self, username, password):
                """Validate user with framework instance given to factory."""
                return wrk.functions['valid_user'](username, password)

        return SecurityMixIn

    def basic_auth_factory(self, wrk):
        """Return a custom barrel-based auth class for the factory instance."""
        outer = self

        class HausBasicAuth(outer.security_mixin, basic.BasicAuth):
            """From haus.factories.BarrelSecurityFactory."""
            pass

        return HausBasicAuth

    def form_auth_factory(self, wrk):
        """Return a custom barrel-based auth class."""
        outer = self

        class HausFormAuth(outer.security_mixin, form.FormAuth):
            """From haus.factories.BarrelSecurityComponent."""

            def username_and_passord(self, environ):
                """Always return None."""
                return None

            def not_authenticated(self, environ, start_response):
                """Use WSGI login app from factory instance's config."""
                return wrk.functions['need_auth'](environ, start_response)

            def cache_username(self, environ, username):
                """Use factory instance's cache_username method."""
                return outer.cache_username(environ, username)

            def get_cached_username(self, environ):
                """Use factory instance's get_cached_username method."""
                return outer.get_cached_username(environ)

        return HausFormAuth

    def combo_auth_factory(self, wrk):
        """Return a custom barrel-based auth class."""
        outer = self

        class HausComboAuth(combo.BasicFormAuth):
            """From haus.factories.BarrelSecurityComponent."""

            basic_auth = outer.basic_auth
            form_auth = outer.form_auth

        return HausComboAuth

    def roles_authz_factory(self, wrk):
        """Return a custom barrel-based auth class for the factory instance."""
        outer = self

        class HausRoles(roles.RolesAuthz):
            """From haus.factories.BarrelSecurityFactory."""

            def user_roles(self, username):
                """Delegate to framework instance."""
                return wrk.functions['user_roles'](username)

            def not_authorized(self, environ, start_response):
                """Use WSGI denied app from framework."""
                return wrk.functions['denied'](environ, start_response)

        return HausRoles

    def init_authn(self, wrk):
        """Set up authentication middleware using factories above."""
        self.security_mixin = self.security_mixin_factory(wrk)
        self.basic_auth = self.basic_auth_factory(wrk)
        self.form_auth = self.form_auth_factory(wrk)
        self.combo_auth = self.combo_auth_factory(wrk)
        self.authn = cooper.decorize(self.combo_auth)
        
    def init_authz(self, wrk):
        """Set up authorization middleware using factories above."""
        self.roles_authz = self.roles_authz_factory(wrk)
        self.authz = cooper.decorize(self.roles_authz)

    def cache_username(self, environ, username):
        """Store the username in a session dict if found.
        
        Also populates REMOTE_USER.
        """
        environ['REMOTE_USER'] = username
        session = environ['haus.functions']['get_session'](environ)
        if session is not None:
            session[self.user_key] = username

    def get_cached_username(self, environ):
        """Look for the username in the session if found.
        
        Also populates REMOTE_USER if it can.
        """
        session = environ['haus.functions']['get_session'](environ)
        if session is not None:
            return session.get(self.user_key)
        else:
            return None

    def user_roles(self, username):
        """Look in config for roles."""
        return self.roles.get(username, [])

    def logout(self, environ):
        """In this case just invalidate the session."""
        return environ['haus.functions']['invalidate_session'](environ)

    @tack(['sessions', 'yaro'])
    def need_auth(self, req):
        """Redirect to login with attempted URI in query string."""
        req.res.headers['Content-type'] = 'text/html'
        return req.redirect(req.uri(self.login_location)
                            +'?uri='+req.uri(with_qs=True))

    @tack(['sessions', 'yaro'])
    def login_page(self, req):
        """."""
        username = req.form.get('username', '')
        password = req.form.get('password', '')
        funcs = req.environ['haus.functions']
        valid_user = funcs['valid_user']
        login = funcs['login']
        get_user = funcs['get_user']
        if req.method == 'POST':
            if valid_user(username, password):
                message = "Login Success!"
                login(req.environ, username)
            else:
                message = "Login failed"
        else:
            message = "Please login"
        _username = get_user(req.environ)
        if _username:
            alternate = req.query.get(
                'uri', 
                req.uri(self.landing_location)
            )
            req.redirect(req.form.get('uri', alternate))
        template_vars = dict(
            message=message,
            username=username,
            layout=self.layout
        )
        render = req.environ['haus.functions']['render_view']
        req.res.headers['Content-type'] = 'text/html'
        return render(self.template_login, template_vars)

    @tack(['sessions', 'yaro'])
    def logout_page(self, req):
        template_vars = dict(
            layout=self.layout
        )
        req.ff['logout'](req.environ)
        render = req.environ['haus.functions']['render_view']
        req.res.headers['Content-type'] = 'text/html'
        return render(self.template_logout, template_vars)

    @tack(['sessions', 'yaro'])
    def access_denied(self, req):
        get_user = req.environ['haus.functions']['get_user']
        template_vars = dict(
            username=get_user(req.environ),
            layout=self.layout
        )
        render = req.environ['haus.functions']['render_view']
        req.res.headers['Content-type'] = 'text/html'
        return render(self.template_denied, template_vars)




"""Command Line Interface
======================

The :command:`haus` command comes with 2 subcommands.

--------------------
:command:`haus init` 
--------------------

- initialize a directory as a haus project:

.. code-block:: bash

    $ haus init --help
    usage: haus init package_name [target_dir]

    options:
      --version   show program's version number and exit
      -h, --help  show this help message and exit

-------------------
:command:`haus run` 
-------------------

- run via wsgiref for development:

.. code-block:: bash

    $ haus run  --help
    usage: haus run package_name host port

    options:
      --version             show program's version number and exit
      -h, --help            show this help message and exit
      -c CONFIG, --configs=CONFIG
                            Haus configuration files (comma separated).
      -r, --reload          Reload the package being run.
      -R RELOADLIST, --reloadlist=RELOADLIST
                            Reload packages that start with (comma seperated).

.. seealso::

   See the Quickstart_ for a breif example.

.. _Quickstart: quickstart.html

"""

import sys
from os.path import abspath
from optparse import OptionParser

from wsgiref.simple_server import make_server
from pkg_resources import iter_entry_points, \
                          Requirement, \
                          resource_filename 

from resolver import resolve
from memento import Assassin
from skel import Builder, StringTemplateFilter

from haus.core import Haus


run_usage = "%prog run package_name host port"


def report_error(parser, message):
    print message
    print
    parser.print_help()
    return 1


def run():
    parser = OptionParser(usage=run_usage, version="haus run 0.0.1")
    parser.add_option('-c', '--configs', dest='config',
                      help='Haus configuration files (comma separated).')
    parser.add_option('-r', '--reload', dest='reload', 
                      action='store_true', default=False,
                      help='Reload the package being run.')
    parser.add_option('-R', '--reloadlist', dest='reloadlist',
                      help='Reload packages that start with (comma seperated).')
    options, args = parser.parse_args()
    if len(args) == 3:
        package_name, host, port = args
        try:
            port = int(port)
        except ValueError, ve:
            return report_error(parser, "Port must be a number not "+repr(port))
        if options.config is None:
            configs = []
        else:
            configs = options.config.split(',')
        if options.reload or options.reloadlist:
            if options.reloadlist:
                to_reload = options.reloadlist.split(',')
            else:
                to_reload = [package_name]
            resolver_expression = "haus.core:Haus('%s'" % package_name
            if configs:
                resolver_expression += ", "
                resolver_expression += \
                    ", ".join("'%s'" % c for c in configs)
            resolver_expression += ")"
            application = Assassin(resolver_expression, to_reload)
        else:
            application = Haus(package_name, *configs)
        server = make_server(host, port, application)
        try:
            server.serve_forever()
        except KeyboardInterrupt, ki:
            print "Cheers, then!"
    else:
        return report_error(parser, "Wrong number of arguments.")


init_usage = "%prog init package_name [target_dir]"


def init():
    parser = OptionParser(usage=init_usage, version="haus init 0.1")
    options, args = parser.parse_args()
    if len(args) == 1:
        package_name = args[0]
        target_dir = '.'
    elif len(args) == 2:
        package_name, target_dir = args
    else:
        parser.print_help()
        return 1
    target_dir = abspath(target_dir)
    resource = Requirement.parse("haus")
    init_template = resource_filename(resource, 'haus/init')

    data = {'package_name': args[0]}
    builder = Builder(data, 
                      filters=[StringTemplateFilter()],
                      ignore=['.hg'])
    dirs, files = builder.build(init_template, target_dir)
    print dirs, "directories created."
    print files, "files written."


def main():
    """This is the Haus CLI."""
    if len(sys.argv) == 1:
        print "For help: haus [command] -h"
        print
        print "commands:"
        for entry_point in iter_entry_points('haus.commands'):
            print " "*4 +entry_point.name
    else:
        command = None
        for entry_point in iter_entry_points('haus.commands', sys.argv.pop(1)):
            command = entry_point.load()
        if command is None:
            print "Command not found."
            print
            return 1
        else:
            return command()


def command():
    sys.exit(main())


if __name__ == '__main__':
    command()



"""haus - A WSGI-centric Web programming framework.

Copyright (C) 2006-2008 Luke Arno - http://lukearno.com/

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to:

The Free Software Foundation, Inc., 
51 Franklin Street, Fifth Floor, 
Boston, MA  02110-1301, USA.

Luke Arno can be found at http://lukearno.com/

"""



""".. _SelectorComponent:

``selector`` -- Request Dispatcher
==================================

Dispatch request based on your :file:`urls.map`
using Selector_

.. _Selector: http://lukearno.com/project/selector/

"""

from selector import Selector

from haus.components.abstract import Component


class SelectorComponent(Component):
    """Terminates a stack with selector for dispatch."""

    consumes = ['status404', 'status405']
    provides = ['get_selector']

    def __init__(self, wrk):
        self.selector = None
        wrk.functions['get_selector'] = self.get_selector

    def get_selector(self, environ):
        """Return the selector instance for this app instance."""
        return self.selector

    def __call__(self, wrk, *args, **kwargs):
        if self.selector is None:
            selector = Selector()
            selector.__name__ = wrk.package_name
            selector.wrap = wrk.stackify
            if 'login_page' in wrk.functions:
                login_page = wrk.functions['login_page']
                selector.add("/login/", GET=login_page, POST=login_page)
            if 'logout_page' in wrk.functions:
                logout_page = wrk.functions['logout_page']
                selector.add("/logout/", GET=logout_page)
            if 'static_app' in wrk.functions:
                static_app = wrk.functions['static_app']
                selector.add("/_static|", GET=static_app, HEAD=static_app)
            mapfile = wrk.pdfilename(str(wrk.config['selector']['map']))
            selector.slurp_file(mapfile)
            selector.status404 = wrk.functions['status404']
            selector.status405 = wrk.functions['status405']
            self.selector = selector
        def middleware(app):
            return self.selector
        return middleware




""".. _LocationsComponent:

``locations`` -- Application Locations
======================================

Find common locations in the application.

"""

from haus.components.abstract import Component


class LocationsComponent(Component):

    provides = ['get_locations']

    def __init__(self, wrk):
        """Set up locations, relative paths considered from prefix."""
        locations = wrk.config.get('locations', {})
        for name, path in locations.items():
            if not path.startswith('/'):
                locations[name] = wrk.config['app']['prefix'] + "/" + path
        self.locations = locations
        wrk.functions['get_locations'] = self.get_locations 

    def get_locations(self, environ):
        return self.locations



""".. _HowHausWorks:

How Haus Works
==============

Haus composes multi-tiered WSGI middleware stacks.
There is an app-wide stack, a trunk stack, if you will,
that receives all incoming requests,
and branch stacks wrapping individual handlers to which 
the top of the trunk stack dispatchs. (You could have
multiple dispatchers and additional tiers of stacked 
middleware, but one is usually enough.)

--------
Stacking
--------

Stacks are created based on descriptors. A descriptor
is just a component name, or a 1, 2, or 3 tuple of
(component_name, args, kwargs)  or a list of stack
descriptors (but not nested). It describes a component
or a stack of components and what to call them with.
The middlewares returned by the components are then
applied in reverse order such that the leftmost
descriptor in a list is the outermost wrapper in the
stack.

.. code-block:: python

    stack = ['component1', ('component2`, ['foo'] {'bar':'baz'})]

----------------------------
Instantiating an Application
----------------------------

At the center of Haus is :class:`haus.core.Haus`. This class is
instantiated with the name or your Haus project (which is a 
just a regular Python package with a certain structure). An 
instance is a WSGI application for your WSGI server of choice. 
All additional arguements should be the names of config files. Haus
looks for configuration in your package, in :file:`mypackage/app.conf`
and merges all addition config files onto that, left to right. Basic 
config validation is done after the configs are parsed; you can 
extend the configuration requirements using the syntax for 
ConfigObj_ in your own :file:`mypackage/configspec.conf`. Then it 
loads the components, in the default order or the order specified 
in the configs. Components are loaded and the trunk stack is built.

.. code-block:: python

    from hause.core import Haus
    application = Haus('yourpackage', *list_of_config_files)

------------------
Loading Components
------------------

Components are found either in the config or through the 
*'haus.components'* entry point via pkg_resources. This means
you can add components with new names or overide the name
of existing component. Since components document the funcitons
they provide and consume in simple lists, creating components
is straight forward.

.. code-block:: ini

    [haus]
    app.stack = "['component1', ('component2`, ['foo'] {'bar':'baz'})]"
    load.order = component1, component2, component3, component4

    [haus.components]
    component4 = mypackage.mymodule:ComponentClass

----------------
Stack Components
----------------

After other components are loaded, composite components
are created using stack descriptors based on configs.
This means that you can combine `sessions` and `cache`,
for instance, and just call it `state`. It would look
like this in your config:

.. code-block:: ini

    [haus.stacks]
    component5 = "['component3', ('component4', ['foo'])]"

-------------------
Framework Functions
-------------------

As components are loaded they will register the functions
they provide in the :attr:`haus.core.Haus.functions`
dictionary. This dictionary is used by other components and
handlers to access funtionality. Most of these take just
environ, some of them are WSGI apps, such as those that 
send HTTP status. These functions are available to handlers,
even if they are WSGI apps, via ``environ['haus.functions']``,
thanks to the **standard** component.
By default, the Yaro component exposes these via ``req.ff``.

.. code-block:: python

    @tack('somestack')
    def isit(req):
        if req.ff == req.environ['haus.functions']:
            return "It is!"
        else:
            return "It is not!"

---------------
The Trunk Stack
---------------

Finally the "app stack" is created by calling the components
listed in the config. This is the app-wide middleware stack
that usually terminates in the dispatcher. The dispatcher
then dispatches to brach stack wrapped handlers.
 
------------------------
Branch Stacks with @tack
------------------------

Your handlers can simply be decorated with 
:func:`haus.core.tack`, which will attach the given
stack descriptor to your handler. A middleware stack
is applied to your handler based on this descriptor
only when it is loaded by the dispather. This keeps
your handlers from getting burried in middleware so 
that you can unit test them.

--------------------------
Application Initialization
--------------------------

If your package has an ``init()`` method, it will be called
by the :ref:`AppInitComponent`.

-----------------------------------
Why the Stacks and not Just Layers?
-----------------------------------

There are going to be some components that need request handler-specific 
parameters: ("allowed_roles" need to be provided per-handler for the 
access control layer to provide the granularity sufficient to achieve
any usefulness. The templating layer needs to know which template to use 
for a given handler, what the "view" is for a given "controller," if you 
will.) Maintaining these verticle links binding stacks of components 
together, through what could otherwise be understood as a horizontally 
layered architecture, is a tax paid  on the luxury of centrallizing 
cross-cutting concerns (like dispatch or security) without surrendering 
to a single, inevitably klunky and/or rigid mega-stack built to rule 
them all.
"""

import sys

from pkg_resources import iter_entry_points, \
                          Requirement, \
                          resource_filename
from configobj import ConfigObj, flatten_errors
from validate import Validator

from resolver import resolve

# This is just a hack for Python2.4 (no functools).
from haus.components.abstract import update_wrapper


def make_spec(filename): 
    return ConfigObj(
        filename,
        encoding='UTF-8',
        list_values=False
    )


def config_error_messages(config, result):
    errors = []
    for entry in flatten_errors(config, result):
        section_list, key, error = entry
        if key is not None:
           section_list.append(key)
        else:
            section_list.append('[missing section]')
        section_string = ', '.join(section_list)
        if error == False:
            error = 'Missing value or section.'
        errors.append(section_string + ' = ' + str(error))
    return errors


def tack(stack_descriptor, *args, **kwargs):
    """Decorate callable with metadata for stackify.

    See :meth:`haus.core.Haus.stackify`.
    """
    def deco(app):
        app.haus_metadata = (stack_descriptor, args, kwargs)
        return app
    return deco


class BadStackDescriptorError(Exception): pass


class Haus(object):
    """The framework."""

    def __init__(self, package_name, *config_files):
        """Load config, load components and create the base app stack."""
        self.package_name = package_name
        self.package_resource = Requirement.parse(self.package_name)
        self.haus_resource = Requirement.parse('haus')
        self.functions = {}
        self.load_config(*config_files)
        self.load_components()
        self.app = self.stack(eval(self.config['haus']['app.stack']))(self)

    def pdfilename(self, filename):
        """Use pkg_resources to get package data filenames."""
        return resource_filename(
            self.package_resource, 
            self.package_name + '/' + filename
        )

    def hauspdfilename(self, filename):
        """Use pkg_resources to get haus package data filenames."""
        return resource_filename(
            self.haus_resource,
            'haus/' + filename
        )

    def load_config(self, *config_files):
        """Load up the app configuration."""
        config_spec = make_spec(self.hauspdfilename('configspec.conf'))
        try:
            config_spec.merge(make_spec(self.pdfilename('configspec.conf')))
        except KeyError, ke:
            pass
        config_files = list(config_files)
        config_files.insert(0, self.pdfilename('app.conf'))
        self.config = reduce(
            lambda x, y: x.merge(y) or x,
            (ConfigObj(f, configspec=config_spec) for f in config_files)
        )
        result = self.config.validate(Validator(), preserve_errors=True)
        if result is True:
            return self.config
        else:
            messages = config_error_messages(self.config, result)
            for message in messages:
                print >> sys.stderr, message
            sys.exit(1)

    def load_components(self):
        """Load the framework components."""
        components = {}
        config_components = self.config.get('haus.components', {})
        config_stacks = self.config.get('haus.stacks', {})
        for name in self.config['haus']['load.order']:
            if name in config_components:
                components[name] = resolve(config_components[name])(self)
            else:
                for entry_point in iter_entry_points('haus.components', name):
                    components[name] = entry_point.load()(self)
        self.components = components
        for name, descriptor_string in config_stacks.items():
            components[name] = self.stack_component(eval(descriptor_string))

    def stack(self, stack_descriptor, *args, **kwargs):
        """Build a middleware that applies."""
        if isinstance(stack_descriptor, (str, unicode, tuple)):
            stack_descriptor = [stack_descriptor]
        def stack_middleware(app):
            """Call with None when loading the base stack"""
            for component in reversed(stack_descriptor):
                if isinstance(component, (str, unicode)):
                    mw = self.components[component](self, *args, **kwargs)
                elif isinstance(component, tuple) and len(component) == 2:
                    mw = self.components[component[0]](self, *component[1], **kwargs)
                elif isinstance(component, tuple) and len(component) == 3:
                    mw = self.components[component[0]](self, *component[1], **component[2])
                else:
                    raise BadStackDescriptorError("Stack: "+str(stack_descriptor)+" Component: "+str(component))
                app = mw(app)
            return app
        return stack_middleware

    def stack_component(self, stack_descriptor):
        """Return a component that produces middleware for the descriptor."""
        def component(wrk, *args, **kwargs):
            return self.stack(stack_descriptor, *args, **kwargs)
        return component

    def stackify(self, app):
        """Wrap with stack according to app.haus_metadata.

        The "metadata" is really 3 tuple that is used to call self.stack().
        This will just fall through if the attribute does not exist.
        """
        if not hasattr(app, 'haus_metadata'):
            return app
        stack_descriptor, args, kwargs = app.haus_metadata
        if args is None:
            args = []
        if kwargs is None:
            kwargs = {}
        newapp = self.stack(stack_descriptor, *args, **kwargs)(app)
        return update_wrapper(newapp, app)

    def __call__(self, environ, start_response):
        """Call the application."""
        return self.app(environ, start_response)



""".. _HTTPStatusComponent:

``status`` -- HTTP Status Handlers
==================================

Provides standard handlers for various HTTP status responses.

"""

from pkg_resources import resource_filename

from haus.components.abstract import Component


class HTTPStatusComponent(Component):

    consumes = ['render_template']
    provides = ['status404', 'status405', 'status500']

    def __init__(self, wrk):
        # TODO: These could also be tuples of package, relative_path
        #       to enable template sharing via pkg_resources?
        # For now, relative path is in this package, otherwise absolute.
        config = wrk.config.get('status.templates', {})
        haus_resource = wrk.haus_resource
        self.template404 = config.get(
            'status404',
            resource_filename(haus_resource, 'haus/html/status404.html')
        )
        self.template405 = config.get(
            'status405',
            resource_filename(haus_resource, 'haus/html/status405.html')
        )
        self.template500 = config.get(
            'status500',
            resource_filename(haus_resource, 'haus/html/status500.html')
        )
        self.layout = config.get(
            'layout',
            wrk.pdfilename('html/_layouts/default.html')
        )
        wrk.functions['status404'] = self.status404
        wrk.functions['status405'] = self.status405
        wrk.functions['status500'] = self.status500

    def status404(self, environ, start_response):
        start_response('404 Not Found', [('Content-type', 'text/html')])
        render = environ['haus.functions']['render_view']
        return render(self.template404, {'layout': self.layout})

    def status405(self, environ, start_response):
        start_response('405 Method Not Supported', [('Content-type', 'text/html')])
        render = environ['haus.functions']['render_view']
        return render(self.template405, {'layout': self.layout})

    def status500(self, environ, start_response):
        start_response('500 An Error Occurred', [('Content-type', 'text/html')])
        render = environ['haus.functions']['render_view']
        return render(self.template500, {'layout': self.layout})




""".. _StaticFilesComponent:

``static`` -- Static Content
============================

Uses  Static_ to serve up static content.
Provides a framework function that gets used
by selector when it is present.

.. _Static: http://lukearno.com/projects/static/

"""

from static import Cling

from haus.components.abstract import Component, update_wrapper


class StaticClingComponent(Component):

    provides = ['static_app']

    def __init__(self, wrk):
        cling = Cling(wrk.pdfilename(wrk.config['static']['root']))
        cling.not_found = wrk.functions['status404']
        cling.method_not_allowed = wrk.functions['status405']
        wrk.functions['static_app'] = cling




""".. _ComponentConcepts:

Component Concepts
==================

Components do pretty much everything in Haus.
Components are passed the configured framework
instance (:class:`haus.core.Haus`) during its 
own initialization and again
whenever it calls the Component instance to generate
middleware for stack composition. Stacking is
explained in detail in :ref:`HowHausWorks`.

Haus encourages the use of :mod:`functools` to 
create well behaved decorators and you can get
them through here when writing components, if
you want the Python 2.4 compatibility.
This module also provides a :func:`update_wrapper` and
:func:`wraps`, which come from :mod:`functools` in 
Python 2.5 and higher, but will provide it's own 
implementation for Python 2.4.

.. seealso::

    See also: :ref:`How Haus Works`
"""

# This little bit is a hack for the sake of Python2.4:

try:
    from functools import wraps, update_wrapper, \
                          WRAPPER_ASSIGNMENTS, WRAPPER_UPDATES
except ImportError, ie:

    WRAPPER_ASSIGNMENTS = ('__module__', '__name__', '__doc__')
    WRAPPER_UPDATES = ('__dict__',)

    def update_wrapper(wrapper, wrapped, assigned=None, updated=None):
        if assigned is None:
            assigned = WRAPPER_ASSIGNMENTS
        if updated is None:
            updated = WRAPPER_UPDATES
        for attr in assigned:
            try:
                setattr(wrapper, attr, getattr(wrapped, attr))
            except:
                pass
        for attr in updated:
            try:
                getattr(wrapper, attr).update(getattr(wrapped, attr))
            except:
                pass
        return wrapper

    def wraps(wrapped, assigned=None, updated=None):
        def proxy(wrapper):
            return update_wrapper(wrapper, wrapped, assigned, updated)
        return update_wrapper(proxy, wrapped)


class HausComponentError(Exception): pass

class Component(object):
    """Abstract base class for haus components."""

    provides = []
    consumes = []

    def __init__(self, wrk):
        """Stuff that happens when the application is loaded.

        Register functions on the framework instance, 
        initialize this component at load time...
        Be aware that often you must load those components 
        which provide framework functions before those which
        consume them. This is a do-nothing by default.
        """

    def __call__(self, wrk, *args, **kwargs):
        """Provide wrapper when individual handler is loaded.

        In other words, this function is used by the stacker
        to get the middleware to stack. How to provide that 
        is totally up to the component. If this is not 
        implemented for a component and you try to stack
        it, it will raise a :class:`HausComponentError`.
        """
        raise HausComponentError(
            "Component %s cannot be stacked."
            % getattr(self, '__name__', self.__class__.__name__)
        )



""".. _AppInitComponent:

``appinit`` -- Application Initialization
=========================================

If your package has an ``init()`` function, this will
try to call it with the :class:`haus.core.Haus` instance.
By default, his is the very last builtin component run.

"""


from haus.components.abstract import Component

from resolver import resolve


class AppInitComponent(Component):

    def __init__(self, wrk):
        try:
            init = resolve(wrk.package_name+":init")
            init(wrk)
        except (ImportError, AttributeError), err:
            pass




""".. _GenshiComponent:

``genshi`` -- Genshi Templating
===============================

Render templates using Genshi_. Assumes that
the name of the template is the same as the name
of your handler.

.. _Genshi: http://genshi.edgewall.org/

"""

from genshi.template import TemplateLoader

from haus.components.abstract import Component, wraps


class GenshiTemplatingComponent(Component):

    provides = ['render_view']

    def __init__(self, wrk):
        config = wrk.config.get('genshi', {})
        package_templates = wrk.pdfilename(config.get('templates.dir', 'html'))
        template_search_path = list(config.get('path', []))
        template_search_path.insert(0, package_templates)
        self.loader = TemplateLoader(
            template_search_path,
            auto_reload=config.get('reload', False)
        )
        wrk.functions['render_view'] = self.render_view

    def render_view(self, template_name, stuff, mode='html', doctype='html', text=False):
        # TODO: Actually handle text templates.
        template = self.loader.load(template_name)
        genned = template.generate(**stuff)
        return genned.render(mode, doctype=doctype)

    def __call__(self, wrk, *args, **kwargs):
        render = wrk.functions['render_view']
        def middleware(app):
            if len(args) == 1:
                template_name = args[0]
            else:
                template_name = "%s.html" % app.__name__
            if kwargs.get('text') == True:
                text = True
            else:
                text = False
            @wraps(app)
            def proxy(environ, start_response):
                stuff = app(environ, start_response)
                stuff.setdefault('config', wrk.functions['get_config'](environ))
                if isinstance(stuff, dict):
                    return render(template_name, stuff, text=text)
                else:
                    return stuff
            return proxy
        return middleware



""".. _StandardFunctionsComponent:

``standard`` -- Standard Functions
==================================

Provides some baseline functions for the framework.
Really just ``routing_args``, ``routing_kwargs`` and some
do-nothings that are there to be overridden by
later-loading components. This is really just plumbing that
most users probably don't care about.

"""

from haus.components.abstract import Component, wraps


class StandardFunctionsComponent(Component):

    provides = [
        'get_session', 
        'invalidate_session', 
        'routing_args', 
        'routing_kwargs',
        'get_cache',
        'get_logger',
        'get_config',
    ]

    def __init__(self, wrk):
        # Should some of these be raising an error "Not Implemented"?
        self.config = wrk.config
        wrk.functions['get_session'] = self.get_session
        wrk.functions['invalidate_session'] = self.invalidate_session
        wrk.functions['routing_args'] = self.routing_args
        wrk.functions['routing_kwargs'] = self.routing_kwargs
        wrk.functions['get_cache'] = self.get_cache
        wrk.functions['get_logger'] = self.get_logger
        wrk.functions['get_config'] = self.get_config

    def get_session(self, environ):
        return None

    def invalidate_session(self, environ):
        return None

    def routing_args(self, environ):
        return environ.get('wsgiorg.routing_args', ((), None))[0]

    def routing_kwargs(self, environ):
        return environ.get('wsgiorg.routing_args', (None, {}))[1]

    def get_cache(self, environ):
        return None

    def get_logger(self, environ):
        return None

    def get_config(self, environ):
        return self.config

    def __call__(self, wrk, *args, **kwargs):
        def middleware(app):
            @wraps(app)
            def proxy(environ, start_response):
                environ['haus.functions'] = wrk.functions
                return app(environ, start_response)
            return proxy
        return middleware



""".. _SessionsandCachingComponent:

``sessions``, ``cache`` -- Sessions and Caching
===============================================

Sessions and caching are provided trough Beaker_,
which provides an excellent array of storage
options. See :ref:`Configuring`.

.. _Beaker: <http://beaker.groovie.org/>

"""

from beaker.middleware import SessionMiddleware, CacheMiddleware

from haus.components.abstract import Component, wraps, update_wrapper


class BeakerSessionsComponent(Component):

    provides = ['get_session', 'invalidate_session']

    def __init__(self, wrk):
        wrk.functions['get_session'] = self.get_session
        wrk.functions['invalidate_session'] = self.invalidate_session
        config = wrk.config.get('sessions', {})
        self.environ_key = \
            str(config.get('environ_key', wrk.package_name + '.session'))
        
    def get_session(self, environ):
        return environ.get(self.environ_key)

    def invalidate_session(self, environ):
        session = self.get_session(environ)
        return session.invalidate()

    def __call__(self, wrk, *args, **kwargs):
        config = wrk.config['sessions']
        #if not config.get('data_dir', '/').startswith('/'):
        #    config['data_dir'] = wrk.pdfilename(config['data_dir'])
        config = dict((str(k), str(v)) for k, v in config.iteritems())
        config.update(kwargs)
        config = dict(("session."+k, v) for k, v in config.iteritems())
        def middleware(app):
            @wraps(app)
            def proxy(environ, start_response):
                # TODO: So, session change during itereation of the iterator
                #       returned (when that is the case) would not be saved?
                session = self.get_session(environ)
                if session is not None and len(session.keys()) == 0:
                    # Make sure a cookie gets sent.
                    session.save()
                try:
                    ret = app(environ, start_response)
                finally:
                    session.save()
                return ret
            newapp = SessionMiddleware(
                proxy, 
                environ_key=self.environ_key,
                config=config
            )
            newapp.__name__ = app.__name__
            return newapp
        return middleware


class BeakerCacheComponent(Component):

    provides = ['get_cache']

    def __init__(self, wrk):
        wrk.functions['get_cache'] = self.get_cache
        config = wrk.config.get('cache', {})
        self.environ_key = str(
            config.get('environ_key', wrk.package_name + '.cache')
        )

    def get_cache(self, environ):
        return environ.get(self.environ_key)

    def __call__(self, wrk, *args, **kwargs):
        config = wrk.config['cache']
        config = dict((str(k), str(v)) for k, v in config.iteritems())
        config.update(kwargs)
        config = dict(("cache."+k, v) for k, v in config.iteritems())
        def middleware(app):
            newapp = CacheMiddleware(
                app, 
                environ_key=self.environ_key, 
                config=config
            )
            newapp.__name__ = app.__name__
            return newapp
            return update_wrapper(newapp, app)
        return middleware




""".. _PrefixerComponent:

``prefixer`` -- Prefix Normalizer
=================================

This somewhat hackish component normalizes the consumption
of the path prefix (the mount point of the applicaiton, 
if you will) from ``environ['PATH_INFO']`` into ``environ['SCRIPT_NAME']``.

"""

from haus.components.abstract import Component, wraps


class PrefixerComponent(Component):
    """Consume the prefix is if seems to be unconsumed."""

    def __init__(self, wrk):
        self.prefix = wrk.config.get('app', {}).get('prefix', '')

    def __call__(self, wrk, *args, **kwargs):
        def middleware(app):
            @wraps(app)
            def proxy(environ, start_response):
                if not environ['SCRIPT_NAME'].endswith(self.prefix) \
                   and environ['PATH_INFO'].startswith(self.prefix):
                    environ['PATH_INFO'] = environ['PATH_INFO'][len(self.prefix):]
                    environ['SCRIPT_NAME'] += self.prefix
                return app(environ, start_response)
            return proxy
        return middleware



"""setup - setuptools based setup for Haus

Copyright (C) 2006-2008 Luke Arno - http://lukearno.com/

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to:

The Free Software Foundation, Inc., 
51 Franklin Street, Fifth Floor, 
Boston, MA  02110-1301, USA.

Luke Arno can be found at http://lukearno.com/

"""

import os

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages

setup(name='Haus',
      version='0.0.2',
      description=\
        'A loosely joined WSGI-centric Web programming framework.',
      long_description="""\
It is easy to add components to the framework or override components 
with your own versions on a per application basis. You can control how 
components are loaded and applied to your application by adding simple 
configuration options. The framework is connected to your application 
code and templates with an easy-to-use decorator.
      """,
      author='Luke Arno',
      author_email='luke.arno@gmail.com',
      url='http://houseofhaus.org/',
      license="LGPL",
      packages=find_packages(),
      install_requires="""
        wsgiref
        Genshi>=0.5
        Beaker
        ConfigObj>=4.5.2
        selector
        static
        yaro
        barrel>=0.1.3
        skel
        resolver
        memento
      """,
      entry_points = """
          [console_scripts]
              haus=haus.cli:main
          [haus.commands]
              init=haus.cli:init
              run=haus.cli:run
          [haus.components]
              prefixer=haus.components.prefixer:PrefixerComponent
              selector=haus.components.selectordispatch:SelectorComponent
              static=haus.components.staticfiles:StaticClingComponent
              yaro=haus.components.yaroreq:YaroComponent
              standard=haus.components.standard:StandardFunctionsComponent
              status=haus.components.status:HTTPStatusComponent
              genshi=haus.components.genshiview:GenshiTemplatingComponent
              logger=haus.components.logger:LoggingComponent
              sessions=haus.components.beakerstate:BeakerSessionsComponent
              cache=haus.components.beakerstate:BeakerCacheComponent
              barrel=haus.components.barrelsec:BarrelSecurityComponent
              locations=haus.components.locations:LocationsComponent
              appinit=haus.components.appinit:AppInitComponent
      """,
      include_package_data=True,
      keywords="wsgi web http framework rest",
      classifiers=['Development Status :: 3 - Alpha',
                   'Environment :: Web Environment',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Software Development :: Libraries',
                   'Topic :: Utilities'])

